@@ -18,10 +18,12 @@
 
 import syslog
 import os
+import socket
 
 def openlog():
 	name = os.getenv( 'LOGNAME', 'unknown' )
-	syslog.openlog( "backintime (%s)" % name )
+	host = socket.gethostname().split('.')[0]
+	syslog.openlog( "%s backintime (%s)" % ( host, name ) )
 
 def closelog():
 	syslog.closelog()
